function op_plotimage(FigNumber,StructureName,VariableToPlot)
% Plots a field in a structure downloaded with OPeNDAP GUIs.
% Inverts the vertical axis, adds a coastline and a colorbar.
%
% op_plotimage(FigureNo,StructureName,VariableToPlot)
%
% INPUT(S):
%
%   FigureNumber -- number to figure in which image will be plotted
%   StructureName -- name of structure in user's workspace
%   VariableToPlot -- (optional) the number of the field to be plotted;
%       if the number for the field to be plotted is absent or if there
%       is more than one field, the user is prompted for the fieldname
%
% OUTPUT(S):
%
%   [none] -- 
%
% NOTE: If the field has more than 2 dimensions, the user is prompted for the
% dimensions to be suppresed.
%
% EXAMPLE:
%
%   PlotImage(2,pathfinder4km,4) will plot the 4th variable in
%   pathfinder4km in figure 2.
%
% OPeNDAP Science Team
% Copyright 2007, 2008
% $Version 2.0.0$

%==========================================================================
% Peter Cornillon
% December 2006
%
% REVISION HISTORY:
% 2006/12/01 created
% 2007/12/01 modified for new data structure
% 2008/02/28 prepare for distribution
%==========================================================================

% Cell array of metadata field names; fields that will not be plotted.

MetadataNames{1} = 'lat';
MetadataNames{2} = 'lon';
MetadataNames{3} = 'metadata';
MetadataNames{4} = 'time';
MetadataNames{5} = 'data';
MetadataNames{6} = 'url';
MetadataNames{7} = 'ce';
MetadataNames{8} = 'name';
MetadataNames{9} = 'request';
MetadataNames{10} = 'temporal';
MetadataNames{11} = 'depth';
MetadataNames{12} = 'readme';
MetadataNames{13} = 'attributes';

NumArguments = nargin;
if NumArguments < 2
    disp(' ')
    disp(' Must supply either 2 or 3 arguments for this function.')
    disp('    First argment is the figure number in which to plot the results.')
    disp('    Second argument is the name of the structure containing the data to be plotted.')
    disp(' Try it again.')
    disp(' ')
    return
end

% Get field names of the structure.

FieldNames = fieldnames( StructureName);   % Get field names first.
FieldNameslc = lower(FieldNames);   % Shift to lower case to avoid case problems in the comparison.
NumFieldNames = length(FieldNameslc);

% Find longitude and latitude vectors. Need these for the plot.

iLatitude = [];
iLongitude = [];
for iNames=1:NumFieldNames
    if ~isempty(findstr(FieldNameslc{iNames},'lat'))
        iLatitude = iNames;
    end
    if ~isempty(findstr(FieldNameslc{iNames},'lon'))
        iLongitude = iNames;
    end
end

if isempty(iLatitude) & isempty(iLongitude)
    disp(' ')
    disp('Missing either a latitude or longitude vector. Can not plot the data')
    disp(' ')
    return
end

% Now get list of plottable data; actually exclude non-plottable fields from list.

iSave = [];
for iNames=1:NumFieldNames
    isave = 1;
    for jNames=1:length(MetadataNames)
        if ~isempty(findstr(FieldNameslc{iNames},MetadataNames{jNames}))
            isave = 0;
        end
    end
    
    % This variable should be plottable; add it to the list.

    if isave == 1
        iSave =[iSave iNames];
    end
end

NumArguments = length(iSave);


% Make sure that a variable to plot has been selected.
if NumArguments == 1
    VariableToPlot = iSave;
else
    disp(' Plottable variables available for this array are')
    FieldNames = fieldnames( StructureName);
    for i=iSave
        disp([i FieldNames(i)])
    end
    VariableToPlot = input('Choose one of these [1, 2,...]')
end

eval(['PlotVariable = cell2mat(FieldNames(' num2str(VariableToPlot) '));'])    % Get the name of the element to plot

eval(['figure(' num2str(FigNo) ')'])

hold off

eval([ 'StructureName.' PlotVariable ' = squeeze(StructureName.' PlotVariable ');'])

eval(['Dimensionality = size(StructureName.' PlotVariable ');'])

if length(Dimensionality) >= 3
    disp([FieldNames(VariableToPlot) ' is a ' num2str(length(Dimensionality)) ' dimenstion array.'])
    disp(['PlotImage will only plot 2-D arrays so you must select a single element for the dimensions that you do not want to plot;'])
    disp([' i.e., you must choose a single value in ' num2str(length(Dimensionality)-2) ' of the dimensions.'])
    Constraint = ['('];
    LastDimension = 1;
    eval(['size(StructureName.' PlotVariable ')'])

    for i=1:length(Dimensionality)-2
        if i == 1
            DimString = 'first';
        end
        if i == 2
            DimString = 'second';
        end
        if i == 3
            DimString = 'third';
        end
        if i == 4
            DimString = 'fourth';
        end
        if i == 5
            DimString = 'fifth';
        end

        CollapseDimension(i) = input(['Specify the ' DimString ' dimension to be collapsed to one element: ']);
        ElementToPlot(i) = input(['Specify the element to be plotted in dimension ' num2str(CollapseDimension(i)) ': ']) ;
        for j=LastDimension:CollapseDimension(i)-1
            Constraint = [Constraint ':,'];
        end
        Constraint = [Constraint num2str(ElementToPlot(i)) ','];
        LastDimension = CollapseDimension(i) + 1;
    end
    Constraint = [Constraint(1:length(Constraint)-1) ')'];
    eval(['imagesc( StructureName.' FieldNames{iLongitude} ', StructureName.' FieldNames{iLatitude} ', squeeze(StructureName.' PlotVariable Constraint '))'])
else
    eval(['imagesc( StructureName.' FieldNames{iLongitude} ', StructureName.' FieldNames{iLatitude} ', StructureName.' PlotVariable ')'])
end
colorbar
set(gca,'ydir','normal')

% Now plot the coast. I think that coast is in image toolbox, so if this
% toolbox is not present, disable the following.

ToolBoxes = ver;
for i=1:length(ToolBoxes)
    if strcmp(ToolBoxes(i).Name,'Mapping Toolbox') == 1   % Mapping toolbox present?

        load coast   % Load the coast.

        eval(['NeedToShift = max(StructureName.' FieldNames{iLongitude} ') > 180; '])

        if NeedToShift   % This section shifts coastline from -180 to 180 to 0 to 360.
            [Lon Lat] = op_shiftcoast(long,lat);
        else
            Lon = long;
            Lat = lat;
        end
        if exist('NoCoast')
            hold on
            plot(Lon,Lat,'w')
        end
    end
end


return
