function varargout = get_hycom(varargin)
% Gets Hybrid Coordinate Ocean Model (HYCOM) data.
%
%  Usage:
%
%  Get_HYCOM  
%
%  without arguments brings up GUI that allows a user to select variables, 
%  time, latitude, longitude, layer limits and save data to workspace or files. 
%  All selections are saved in a cell array Request.
%
%  Get_HYCOM(Request)
%
%  gets data selected by a prior call which was saved in the cell array Request.
%
%  Files: Get_HYCOM.m      - main function, this file
%         GUI_HYCOM.m      - GUI function
%         GUI_HYCOM.fig    - GUI graphic window
%         GUI_HYCOM_Request.mat    - contains the last Request
%  
% OPeNDAP Science Team
% Copyright 2007
% $Version 2.1.0$

%==========================================================================
% Meri Sheremet
% September 2007
%
% REVISION HISTORY:
% 2007/01/01 created, ms
%==========================================================================

%Old HYCOM websites
%URLBase='http://hycom.rsmas.miami.edu/cgi-bin/nph-dods/datasets/ocean-prediction/NAT/best-estimate';
%URLBase='http://asterix.rsmas.miami.edu/cgi-bin/nph-dods/datasets/ocean-prediction/atlantic/best-estimate';
%URLBase='http://hycom.coaps.fsu.edu/opendap/nph-dods/datasets/ocean-prediction/atlantic/best-estimate';
%URLBase='http://hycom.coaps.fsu.edu/opendap/nph-dods/datasets/hycom/atlantic/ATLd0.08_native_coordinate_data/best-estimate/';
URLBase='http://hycom.coaps.fsu.edu/opendap/nph-dods/datasets/hycom/atlantic/ATLd0.08/data';

FileNamePrefix='archv.';FN0=FileNamePrefix;
MBLIMIT=100; % warning if requested amount of data > MBLIMIT

% Get HYCOM dataset dates from the list of files in the dens directory
global DataSetDATES 
global DirList
if ~exist('DataSetDATES','var') | isempty(DataSetDATES)
% get list of dates from the list of files
URL=[URLBase '/' 'dens' '/'];
h=msgbox('Acquiring metadata from HYCOM site, please wait ...');
% next line makes OK button invisible
hh=get(h,'Children'); set(hh(2),'Visible','off'); pause(1);
[DirList,URLReadStatus]=urlread(URL); 
close(h)
    if URLReadStatus == 0
msgbox('Could not access HYCOM site. Sorry, try again later.'); return
    end 
nn = findstr(DirList,[FileNamePrefix]);
mm=length([FileNamePrefix]);
DataSetDATES = [];
    for k=1:length(nn)
DataSetDATES = [DataSetDATES; DirList(nn(k)+mm:nn(k)+mm+7)];
    end
DataSetDATES=DataSetDATES(1:2:end,:); % one date has two entries *.nc.html and *.nc
[L1,L2]=size(DataSetDATES);jvalid=zeros(L1,1);
for j=1:L1
jvalid(j)=~isempty(str2num(DataSetDATES(j,:)));
end
jvalid=find(jvalid==1);
DataSetDATES=DataSetDATES(jvalid,:);
evalin('base','global DataSetDATES')
end % ~exist('DataSetDATES','var') | isempty(DataSetDATES)
DATES=DataSetDATES;

if nargin<1
% no arguments passed, get parameters from GUI     
h = gui_hycom;
if nargout>0
    varargout{1} = h; %handle is output.
end
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
% one argument Request, get HYCOM data  

% Example of a loaddap data request:
% loaddap('http://hycom.rsmas.miami.edu/cgi-bin/nph-dods/datasets/
% ocean-prediction/NAT/best-estimate/temp/hycom_2.1_nat_1o12ml_temp_20030601.nc?
% temp[0:1:25][0:1:1608][0:1:1677]')

request=varargin{1}; R=request;
assignin('base','request',request);

if ~exist('loaddap')
errordlg('loaddap not found. Make sure it is in your MATLAB path.'); return
end               
if length(R.DIRNAME)>0 & ~exist(R.DIRNAME);   
eval(['mkdir ' R.DIRNAME]);    
end
if sum(strcmp(R.Fields(:,2),'y'))==0 
errordlg('No variables selected.'); return
end
[L1,L2]=size(R.Fields);

ArraysSameSize=1;
if ArraysSameSize==1
% HYCOM variables are defined on a staggered grid.
% Corresponding lat, lon arrays may have different values.
% Make sure that the index ranges of all requested variables are same.
% Calculate index ranges based on dens, a variable defined at the center of a cell.
VariableName='dens';
FileNameSuffix='_3zr';
DATE0=DATES(1,:); % 
if exist('Latitude') 
clear Latitude; 
end
URL2=[URLBase '/' VariableName '/' FileNamePrefix DATE0 '00' FileNameSuffix '.nc'];
URL3=[URL2 '?Longitude,Latitude,Depth'];
loaddap('+v',URL3 );
if ~exist('Latitude') % files may be compressed, extension .gz 
URL3=[URL2 '.gz' '?Longitude,Latitude,Depth'];
loaddap('+v',URL3);
end

iDATE1=min(find(str2num(DATES) >= str2num(R.DATE1)));
iDATE2=max(find(str2num(DATES) <= str2num(R.DATE2)));
if(isempty(iDATE1) | isempty(iDATE2) | (iDATE1>iDATE2)) 
errordlg('Requested dates are out of range.'); return
end

if isempty(R.LAT1) | isempty(R.LAT2) | isempty(R.LON1) | isempty(R.LON2)
    errordlg(['Latitude/Longitude values are invalid or empty.']);
    return
end
% find indices within the specified range of lons, lats
iLON1=min(find(Longitude >= R.LON1))-1;
iLON2=max(find(Longitude <= R.LON2))-1;
if(isempty(iLON1) | isempty(iLON2) | isempty(R.LONINCR) | (iLON1>iLON2)) 
errordlg('Requested longitudes are out of range.'); return
end
iLAT1=min(find(Latitude >= R.LAT1))-1;
iLAT2=max(find(Latitude <= R.LAT2))-1;
if(isempty(iLAT1) | isempty(iLAT2) | (iLAT1>iLAT2)) 
errordlg('Requested latitudes are out of range.'); return
end
end

if(isempty(R.LATINCR)|isempty(R.LONINCR)|isempty(R.DATEINCR)|R.LATINCR==0|R.LONINCR==0|R.DATEINCR==0) 
    errordlg(['Subsampling values are invalid.']);
    return
end

Zstd=Depth;
% 33 standard levels 
% find range of indices for depth levels
if (sum(strcmp(R.Fields(1:5,2),'y')) > 0) % at least one 4D field is checked
    if (isempty(R.DEPTH1) | isempty(R.DEPTH2)) 
    errordlg(['Depth levels values are invalid or empty.']); return
    end
iLEVEL1=min(find(Depth >= R.DEPTH1))-1;
iLEVEL2=max(find(Depth <= R.DEPTH2))-1;
    if (isempty(iLEVEL1) | isempty(iLEVEL2) | isempty(R.LEVELINCR) ...
   |(iLEVEL1<0)|(iLEVEL2<0)|(iLEVEL1>32)|(iLEVEL2>32)|(iLEVEL1>iLEVEL2))    
errordlg('Requested depth levels are out of range.'); return
    end
else 
iLEVEL1=0;
iLEVEL2=0;
R.LEVELINCR=1;
end

%estimate the amount of requested data
kV4=sum(strcmp(R.Fields(1:5,2),'y')); % number of selected 4D fields 
kV3=sum(strcmp(R.Fields(6:17,2),'y')); % number of selected 3D fields 
kDAT=(iDATE2-iDATE1)/R.DATEINCR+1;
kLAT=(iLAT2-iLAT1)/R.LATINCR+1;
kLON=(iLON2-iLON1)/R.LONINCR+1;
kLEV=(iLEVEL2-iLEVEL1)/R.LEVELINCR+1;
MB=round(kDAT*kLAT*kLON*(kV4*kLEV+kV3)*8/1024/1024);
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'Requested Data Amount','Continue','Cancel','Cancel');
    if strcmp(ButtonName,'Cancel') return; end
end

if strcmp(R.SaveMode,'opendap')==1 
% find the latest saved opendap frame number
[kw0,kf0]=LastFrameNum(R);
end
kf=0;

hwaitbar=waitbar(0,'Acquiring ','Name','HYCOM Download Progress');
for k=iDATE1:R.DATEINCR:iDATE2 % loop through dates
clear D % standard opendap_nnnn data structure containing a single time frame
kf=kf+1;     
DATE=DATES(k,:);

kVV=0;
    for kV=1:L1 % loop through checked variables
        if R.Fields{kV,2}=='y'
VariableName=R.Fields{kV,1};
VariableDim=R.Fields{kV,3};
VariableDir=R.Fields{kV,4};
FileNameSuffix=R.Fields{kV,5};
kVV=kVV+1;
clear(VariableName)

if VariableDim==4
URL2=[URLBase '/' VariableDir '/' FileNamePrefix DATE '00' FileNameSuffix '.nc'];
else
URL2=[URLBase '/' VariableDir '/' FileNamePrefix DATE '00' FileNameSuffix '.nc'];
end

xprogr=((kVV-1)+(k-iDATE1)/(iDATE2-iDATE1+1))/(kV4+kV3); % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' strrep(VariableName,'_',' ') ' ' DATE],'Name','HYCOM Download Progress');

%check if file is zipped or not
clear Latitude
URL=[URL2 '.gz'];
URL3=[URL '?Latitude[0:1:0]'];
loaddap('+v',URL3);
if ~exist('Latitude')
URL=URL2;
end
clear Latitude

C0='[0:1:0]'; % constraint time 
C1=['[' num2str(iLEVEL1) ':' num2str(R.LEVELINCR) ':' num2str(iLEVEL2) ']']; % Constraint LEVEL
C2=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % Constraint Latitude
C3=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % Constraint Longitude

clear Latitude
if VariableDim == 4
URL=[URL '?' VariableName C0 C1 C2 C3 ]; % 4D variables
else 
URL=[URL '?' VariableName C0 C2 C3 ];    % 3D variables
end
disp(URL)
loaddap('+v',URL)

% replace missing values with NaN
A=loaddap('-A',URL); 
%MV=eval(['A.' VariableName '.missing_value']);
eval(['nn = find(' VariableName '.' VariableName '>1.0e+30);'])
eval([VariableName '.' VariableName '(nn) = NaN;'])

% add URL for future reference
eval([ VariableName '.url=URL;' ])
% add attributes as well 
eval([ VariableName '.attributes=A;' ])

if R.SaveWorkspace == 'y'& strcmp(R.SaveMode,'native')==1
%% save extracted data in workspace variables
eval(['tmp=' VariableName ';']);
% if you use a letter t instead of tmp,
% a mysterious window with a circle pops up 
%t
assignin('base',[VariableName '_' DATE],tmp)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'native')==1
% save extracted data to files
FN3=[R.FNPREFIX VariableName '_' DATE '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3
save(FN3,VariableName,'URL')
end

        end % if checked variables
    end % kV list of variables

if strcmp(R.SaveMode,'opendap')==1
clear D; % create standard opendap_nnnn data structure

if exist('density','var')                           D.density=density.density; end
if exist('salinity','var')                          D.salinity=salinity.salinity; end   
if exist('temperature','var')                       D.temperature=temperature.temperature; end
if exist('u','var')                                 D.u=u.u; end
if exist('v','var')                                 D.v=v.v; end
if exist('mixed_layer_density','var')               D.mixed_layer_density=mixed_layer_density.mixed_layer_density; end
if exist('mixed_layer_salinity','var')              D.mixed_layer_salinity=mixed_layer_salinity.mixed_layer_salinity; end
if exist('mixed_layer_temperature','var')           D.mixed_layer_temperature=mixed_layer_temperature.mixed_layer_temperature; end
if exist('mixed_layer_thickness','var')             D.mixed_layer_thickness=mixed_layer_thickness.mixed_layer_thickness; end
if exist('mixed_layer_u_velocity','var')            D.mixed_layer_u_velocity=mixed_layer_u_velocity.mixed_layer_u_velocity; end
if exist('mixed_layer_v_velocity','var')            D.mixed_layer_v_velocity=mixed_layer_v_velocity.mixed_layer_v_velocity; end
if exist('surface_salinity_trend','var')            D.surface_salinity_trend=surface_salinity_trend.surface_salinity_trend; end
if exist('surface_temperature_trend','var')         D.surface_temperature_trend=surface_temperature_trend.surface_temperature_trend; end
if exist('ssh','var')                               D.ssh=ssh.ssh; end
if exist('surface_boundary_layer_thickness','var')  D.surface_boundary_layer_thickness=surface_boundary_layer_thickness.surface_boundary_layer_thickness; end
if exist('qtot','var')                              D.qtot=qtot.qtot; end
if exist('emp','var')                               D.emp=emp.emp; end

% variables latitude,longitude,depth
% 3D fields
if exist('density','var')     
    D.latitude=density.Latitude;    
    D.longitude=density.Longitude;
    D.depth=density.Depth;
end
if exist('salinity','var')      
    D.latitude=salinity.Latitude;
    D.longitude=salinity.Longitude;
    D.depth=salinity.Depth;
end   
if exist('temperature','var')   
    D.latitude=temperature.Latitude; 
    D.longitude=temperature.Longitude; 
    D.depth=temperature.Depth; 
end
if exist('u','var')             
    D.latitude=u.Latitude; 
    D.longitude_u=u.Longitude; 
    D.depth=u.Depth; 
end
if exist('v','var')             
    D.latitude_v=v.Latitude; 
    D.longitude=v.Longitude; 
    D.depth=v.Depth; 
end
% 2D fields
if exist('mixed_layer_density','var')       
    D.latitude=mixed_layer_density.Latitude; 
    D.longitude=mixed_layer_density.Longitude; 
end
if exist('mixed_layer_salinity','var')      
    D.latitude=mixed_layer_salinity.Latitude; 
    D.longitude=mixed_layer_salinity.Longitude; 
end
if exist('mixed_layer_temperature','var')   
    D.latitude=mixed_layer_temperature.Latitude;
    D.longitude=mixed_layer_temperature.Longitude;
end
if exist('mixed_layer_thickness','var')     
    D.latitude=mixed_layer_thickness.Latitude; 
    D.longitude=mixed_layer_thickness.Longitude; 
end
if exist('mixed_layer_u_velocity','var')    
    D.latitude=mixed_layer_u_velocity.Latitude;
    D.longitude_u=mixed_layer_u_velocity.Longitude;
end
if exist('mixed_layer_v_velocity','var')    
    D.latitude_v=mixed_layer_v_velocity.Latitude; 
    D.longitude=mixed_layer_v_velocity.Longitude; 
end
if exist('surface_salinity_trend','var')    
    D.latitude=surface_salinity_trend.Latitude; 
    D.longitude=surface_salinity_trend.Longitude; 
end
if exist('surface_temperature_trend','var') 
    D.latitude=surface_temperature_trend.Latitude; 
    D.longitude=surface_temperature_trend.Longitude; 
end
if exist('ssh','var')                       
    D.latitude=ssh.Latitude; 
    D.longitude=ssh.Longitude; 
end
if exist('surface_boundary_layer_thickness','var') 
    D.latitude=surface_boundary_layer_thickness.Latitude; 
    D.longitude=surface_boundary_layer_thickness.Longitude; 
end
if exist('qtot','var')                      
    D.latitude=qtot.Latitude;
    D.longitude=qtot.Longitude;
end
if exist('emp','var')                 
    D.latitude=emp.Latitude; 
    D.longitude=emp.Longitude; 
end

datetime=[DATE '000000'];
%seconds since 1970;
D.time=86400*(datenum(datetime,'yyyymmddHHMMSS')-datenum('19700101000000','yyyymmddHHMMSS')); 

if exist('density','var')                           D.metadata.density_variables={'latitude','longitude','depth'}; end
if exist('salinity','var')                          D.metadata.salinity_variables={'latitude','longitude','depth'}; end
if exist('temperature','var')                       D.metadata.temperature_variables={'latitude','longitude','depth'}; end
if exist('u','var')                                 D.metadata.u_variables={'latitude','longitude_u','depth'}; end
if exist('v','var')                                 D.metadata.v_variables={'latitude_v','longitude','depth'}; end
if exist('mixed_layer_density','var')               D.metadata.mixed_layer_density_variables={'latitude','longitude'}; end    
if exist('mixed_layer_salinity','var')              D.metadata.mixed_layer_salinity_variables={'latitude','longitude'}; end
if exist('mixed_layer_temperature','var')           D.metadata.mixed_layer_temperature_variables={'latitude','longitude'}; end
if exist('mixed_layer_thickness','var')             D.metadata.mixed_layer_thickness_variables={'latitude','longitude'}; end
if exist('mixed_layer_u_velocity','var')            D.metadata.mixed_layer_u_velocity_variables={'latitude','longitude_u'}; end
if exist('mixed_layer_v_velocity','var')            D.metadata.mixed_layer_v_velocity_variables={'latitude_v','longitude'}; end
if exist('surface_salinity_trend','var')            D.metadata.surface_salinity_trend_variables={'latitude','longitude'}; end
if exist('surface_temperature_trend','var')         D.metadata.surface_temperature_trend_variables={'latitude','longitude'}; end
if exist('ssh','var')                               D.metadata.ssh_variables={'latitude','longitude'}; end
if exist('surface_boundary_layer_thickness','var')  D.metadata.surface_boundary_layer_thickness_variables={'latitude','longitude'}; end
if exist('qtot','var')                              D.metadata.qtot_variables={'latitude','longitude'}; end
if exist('emp','var')                               D.metadata.emp_variables={'latitude','longitude'}; end

if exist('density','var')                           D.metadata.url.density=density.url; end
if exist('salinity','var')                          D.metadata.url.salinity=salinity.url; end   
if exist('temperature','var')                       D.metadata.url.temperature=temperature.url; end
if exist('u','var')                                 D.metadata.url.u=u.url; end
if exist('v','var')                                 D.metadata.url.v=v.url; end
if exist('mixed_layer_density','var')               D.metadata.url.mixed_layer_density=mixed_layer_density.url; end
if exist('mixed_layer_salinity','var')              D.metadata.url.mixed_layer_salinity=mixed_layer_salinity.url; end
if exist('mixed_layer_temperature','var')           D.metadata.url.mixed_layer_temperature=mixed_layer_temperature.url; end
if exist('mixed_layer_thickness','var')             D.metadata.url.mixed_layer_thickness=mixed_layer_thickness.url; end
if exist('mixed_layer_u_velocity','var')            D.metadata.url.mixed_layer_u_velocity=mixed_layer_u_velocity.url; end
if exist('mixed_layer_v_velocity','var')            D.metadata.url.mixed_layer_v_velocity=mixed_layer_v_velocity.url; end
if exist('surface_salinity_trend','var')            D.metadata.url.surface_salinity_trend=surface_salinity_trend.url; end
if exist('surface_temperature_trend','var')         D.metadata.url.surface_temperature_trend=surface_temperature_trend.url; end
if exist('ssh','var')                               D.metadata.url.ssh=ssh.url; end
if exist('surface_boundary_layer_thickness','var')  D.metadata.url.surface_boundary_layer_thickness=surface_boundary_layer_thickness.url; end
if exist('qtot','var')                              D.metadata.url.qtot=qtot.url; end
if exist('emp','var')                               D.metadata.url.emp=emp.url; end

if exist('density','var')                           D.metadata.attributes.density=density.attributes; end
if exist('salinity','var')                          D.metadata.attributes.salinity=salinity.attributes; end   
if exist('temperature','var')                       D.metadata.attributes.temperature=temperature.attributes; end
if exist('u','var')                                 D.metadata.attributes.u=u.attributes; end
if exist('v','var')                                 D.metadata.attributes.v=v.attributes; end
if exist('mixed_layer_density','var')               D.metadata.attributes.mixed_layer_density=mixed_layer_density.attributes; end
if exist('mixed_layer_salinity','var')              D.metadata.attributes.mixed_layer_salinity=mixed_layer_salinity.attributes; end
if exist('mixed_layer_temperature','var')           D.metadata.attributes.mixed_layer_temperature=mixed_layer_temperature.attributes; end
if exist('mixed_layer_thickness','var')             D.metadata.attributes.mixed_layer_thickness=mixed_layer_thickness.attributes; end
if exist('mixed_layer_u_velocity','var')            D.metadata.attributes.mixed_layer_u_velocity=mixed_layer_u_velocity.attributes; end
if exist('mixed_layer_v_velocity','var')            D.metadata.attributes.mixed_layer_v_velocity=mixed_layer_v_velocity.attributes; end
if exist('surface_salinity_trend','var')            D.metadata.attributes.surface_salinity_trend=surface_salinity_trend.attributes; end
if exist('surface_temperature_trend','var')         D.metadata.attributes.surface_temperature_trend=surface_temperature_trend.attributes; end
if exist('ssh','var')                               D.metadata.attributes.ssh=ssh.attributes; end
if exist('surface_boundary_layer_thickness','var')  D.metadata.attributes.surface_boundary_layer_thickness=surface_boundary_layer_thickness.attributes; end
if exist('qtot','var')                              D.metadata.attributes.qtot=qtot.attributes; end
if exist('emp','var')                               D.metadata.attributes.emp=emp.attributes; end

D.metadata.name='hycom';
D.metadata.temporal='time_series';
D.metadata.reference_time=datestr([1970,01,01,00,00,00],'yyyy-mm-dd HH:MM:SS');
D.metadata.readme='time seconds since reference_time; missing values replaced with NaN';
D.metadata.user_friendly_time=datestr(datenum(datetime,'yyyymmddHHMMSS'),'yyyy-mm-dd HH:MM:SS');
D.metadata.request_date = R.RequestDate;
D.metadata.request=R; 
end

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==1
KF=num2str(kf+kw0,'%04d');
WSN=['opendap_' KF];
assignin('base',WSN,D)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
% save extracted data to files
KF=num2str(kf+kf0,'%04d');
FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D')
end

end % loop through dates
xprogr=1;
waitbar( xprogr , hwaitbar, ['Acquiring ' strrep(VariableName,'_',' ') ' ' DATE],'Name','HYCOM Download Progress');
close(hwaitbar)
%msgbox('The data have been acquired.')
disp('The data have been acquired.')
end % nargin

%=====================================
function [kw,kf]=LastFrameNum(R)
% find the latest saved opendap frame number 
%KF=num2str(kf,'%04d');
% the latest workspace frame number of form opendap_nnnn
a=evalin('base','who');k=strmatch('opendap_',a);
if isempty(k) k=0; 
else 
a=a{k(end)};
ki=strfind(a,'_');
    if length(ki)==1 ki(2)=length(a)+1; end
k=str2num(a(ki(1)+1:ki(2)-1));
end
if isempty(k) k=0;
end
kw=k;
% the latest saved file of form FNPREFIXnnnn.mat
if isempty(R.DIRNAME) R.DIRNAME='.'; end
%[R.DIRNAME '\' R.FNPREFIX '*.mat']
a=dir([R.DIRNAME '\' R.FNPREFIX '*.mat']);
a={a.name};
if isempty(a) k=0;
else
a=a{end};
k=str2num(a(length(R.FNPREFIX)+1:end-4));
end
if isempty(k) k=0;
end
kf=k;
