function varargout = get_oaflux(varargin)
% Gets Objectively Analyzed Air-Sea Flux (OAFlux) data.
%
% get_oaflux
% get_oaflux(request)
%
% INPUT(S):
%
%   request -- (optional) structure input which contains
%              information necessary to obtain data. The GUI
%              merely provides a graphical method of generating
%              the "request" variable. However, this visual
%              interface may be by-passed by this method.
%
% OUTPUT(S):
%
%   [none]
%
% OPeNDAP Science Team
% Copyright 2007
% $Version 2.1.0$

%==========================================================================
% Meri Sheremet
% May 2007
%
% REVISION HISTORY:
% 2007/01/01 created, ms
% 2007/09/20 opendap_nnnn structure finalized
%==========================================================================

% Dataset summary
% 
% http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/daily/whoi_oaflux_rad_heat_d
% http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/monthly/whoi_oaflux_rad_heat_m
% nswrs(time,lat,lon) 
% nlwrs(time,lat,lon) 
%    00Z01JUL1983 to 00Z31DEC2002 (7124 points, avg. res. 1.0 days)
%    00Z15JUL1983 to 00Z15DEC2002 (234 points, avg. res. 30.442 days)
% http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/daily/whoi_oaflux_turb_heat_d
% http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/monthly/whoi_oaflux_turb_heat_m
% lhtfl(time,lat,lon)
% shtfl(time,lat,lon)
%    00Z01JAN1981 to 00Z31DEC2002 (8035 points, avg. res. 1.0 days)
%    00Z15JAN1981 to 00Z15DEC2002 (264 points, avg. res. 30.433 days)
% Longitude: 0.5E to 359.5E (360 points, avg. res. 1.0)
% Latitude: -64.5N to 64.5N (130 points, avg. res. 1.0)
% [time][lat][lon] in loaddap request
% (lat,lon,time) in MATLAB structure

if nargin<1
% no arguments passed, get parameters from GUI     
h = gui_oaflux;
if nargout
    varargout{1} = h; %set figure handle as output
end
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
% one argument request, get OAFlux data  

URLBase='http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/';
MBLIMIT=100;

request=varargin{1};
R=request;

assignin('base','request',request);
   
if~exist('loaddap')
  errordlg(['loaddap not found. Make sure it is in your MATLAB path.'])
return
end
        
if isempty(R.LAT1) | isempty(R.LAT2) | isempty(R.LON1) | isempty(R.LON2)
    errordlg(['Latitude/Longitude values are invalid or empty.']);
    return
end

Longitude=(0.5:1:359.5)';
Latitude=(-64.5:1:64.5)';

% Find indices within the specified range of lons, lats
iLAT1=min(find(Latitude >= R.LAT1))-1;
iLAT2=max(find(Latitude <= R.LAT2))-1;
iLON1=min(find(Longitude >= R.LON1))-1;
iLON2=max(find(Longitude <= R.LON2))-1;


if(isempty(iLAT1) | isempty(iLAT2)) | (iLAT2<iLAT1)
    errordlg(['Requested latitudes are out of range.']);
    return
end
if(isempty(iLON1) | isempty(iLON2)) | (iLON2<iLON1)
    errordlg(['Requested longitudes are out of range.']);
    return
end
if(isempty(R.LATINCR)|isempty(R.LONINCR)) 
    errordlg(['Subsampling values are invalid.']);
    return
end
if(isempty(R.DATE1) | isempty(R.DATE2)) | (str2num(R.DATE2)<str2num(R.DATE1))
    errordlg(['Selected dates are invalid.']);
    return
end

if R.SaveWorkspace=='n' & R.SaveFiles=='n'
    errordlg(['Select how to save data.']);
    return
end
if length(R.DIRNAME) > 0
    if exist(R.DIRNAME)==0
        eval(['mkdir ' R.DIRNAME])
    end    
end


CLAT=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % ConstraintLatitude
CLON=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % ConstraintLongitude
JD1=datenum(R.DATE1,'yyyymmdd');
JD2=datenum(R.DATE2,'yyyymmdd');

% make sure that both dataset requests have same time coverage 
if (R.Fields{1,2}=='y'|R.Fields{2,2}=='y')&(R.Fields{3,2}=='y'|R.Fields{4,2}=='y') & str2num(R.DATE1)<19830701
errordlg(['The radiative heat fluxes (nswrs, nlwrs) have values starting from 1983-07-01. ' ... 
          'To request both the radiative and turbulent (lhtfl, shtfl) ',...
          'heat fluxes enter dates after that one.']);
    return
end

[L1,L2]=size(R.Fields);

% Estimate the amount of requested data
kV=0;
for k=1:L1
    if R.Fields{k,2}=='y' kV=kV+1; end % number of requested variables
end
% kV number of checked fields

if strcmp(R.SaveMode,'opendap')==1 
% find the latest saved opendap frame number
[kw0,kf0]=LastFrameNum(R);
end



clear D
% Progress bar
hwaitbar=waitbar(0,'Acquiring ','Name','OAFlux Download Progress');

for k=1:L1
% select only checked variables
    if R.Fields{k,2}=='y';
field=R.Fields{k,1};        
 
s=field;
xprogr=(k-1)/L1; % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','OAFlux Download Progress');

URLBase='http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/';

%http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/daily/whoi_oaflux_rad_heat_d
%http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/monthly/whoi_oaflux_rad_heat_m
%nswrs(time,lat,lon) 
%nlwrs(time,lat,lon)
%http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/daily/whoi_oaflux_turb_heat_d
%http://apdrc.soest.hawaii.edu:80/dods/public_data/WHOI_OAFlux/monthly/whoi_oaflux_turb_heat_m
%lhtfl(time,lat,lon)
%shtfl(time,lat,lon)

        RADTURB='';
        if field(3)=='w' RADTURB='rad_heat'; end
        if field(3)=='t' RADTURB='turb_heat'; end
URL1=[URLBase R.TAVG '/whoi_oaflux_' RADTURB '_' R.TAVG(1)];
loaddap('+v',[URL1 '?' field '.time'])
% time in days since 1-1-1 in the Dataset
if ~exist('time','var')
    %disp(dods_err_msg)
    msgbox('Could not access site. Sorry, try again later.');
    return
end

ttime=time+365; % convert to days since 0-1-1 as opposed to 1-1-1 in the Dataset
if R.TAVG(1)=='d' % daily
JDA=ttime; 
JDB=JDA;
else % monthly
%first day of the month
JDA=datenum(str2num(datestr(ttime,'yyyy')),str2num(datestr(ttime,'mm')),1);
%last day of the month
JDB=datenum(str2num(datestr(ttime,'yyyy')),str2num(datestr(ttime,'mm'))+1,1)-1;
end

iDATE1=min(find(JDB >= JD1))-1;
iDATE2=max(find(JDA <= JD2))-1;
CDATE=['[' num2str(iDATE1) ':' num2str(R.DATEINCR) ':' num2str(iDATE2) ']']; % Constraint DATE

% Estimate the amount of requested data
MB=round((iDATE2-iDATE1+1)/R.DATEINCR*(iLAT2-iLAT1+1)/R.LATINCR*(iLON2-iLON1+1)/R.LONINCR*kV*8/1024/1024);
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'OAFlux Requested Data Amount','Continue','Cancel','Cancel');
    if strcmp(ButtonName,'Cancel') 
    close(hwaitbar)
    return; 
    end
end
MBLIMIT=1000000; % check no more for the other fields

URL=[URL1 '?' field CDATE CLAT CLON ];
disp(URL)
A=loaddap('-A',URL);
loaddap('+v',URL)
eval(['latitude=' field '.lat;']);
eval(['longitude=' field '.lon;']);
eval(['time=' field '.time;']);
% convert to days since 0-1-1 as opposed to 1-1-1 in the Dataset
ttime=time+365; 
time=(ttime-datenum(1970,1,1,0,0,0))*86400; % seconds since 1970

eval(['D.' field '=' field ';']);
% make cell arrays for URL and Attributes (different for different fields)
%if exist('url') url=[url; URL]; else url=URL; end
eval(['url.' field '=URL;']);
eval(['attributes.' field '=A;']);
    end
end
D.latitude=latitude;
D.longitude=longitude;
% lat,lon,time are supposed to be the same for all fields

% replace missing values with NaN
MV=1.0000e+20; %missing value
MV=MV*0.95;
    if isfield(D,'nlwrs')==1
        D.nlwrs.nlwrs(find(D.nlwrs.nlwrs>MV))=NaN;
    end
    if isfield(D,'nswrs')==1
        D.nswrs.nswrs(find(D.nswrs.nswrs>MV))=NaN;
    end
    if isfield(D,'lhtfl')==1
        D.lhtfl.lhtfl(find(D.lhtfl.lhtfl>MV))=NaN;
    end
    if isfield(D,'shtfl')==1
        D.shtfl.shtfl(find(D.shtfl.shtfl>MV))=NaN;
    end
    
if R.SaveWorkspace == 'y' & strcmp(R.SaveMode,'native')==1 
assignin('base','OAFlux',D)
end

if strcmp(R.SaveMode,'opendap')==1 % split data into time frames
readme = '';
D.time=time;

    if isfield(D,'nlwrs')==1 D.metadata.nlwrs_variables={'latitude','longitude','time'}; end
    if isfield(D,'nswrs')==1 D.metadata.nswrs_variables={'latitude','longitude','time'}; end
    if isfield(D,'lhtfl')==1 D.metadata.lhtfl_variables={'latitude','longitude','time'}; end
    if isfield(D,'shtfl')==1 D.metadata.shtfl_variables={'latitude','longitude','time'}; end

D.metadata.url=url;
D.metadata.attributes=attributes;
D.metadata.name='oaflux';
D.metadata.temporal='time_series';
D.metadata.reference_time=datestr([1970,01,01,00,00,00],'yyyy-mm-dd HH:MM:SS');
readme=['time in seconds since reference_time; /' ...
        'originally was in days since 0001-01-01 /' ...
        'missing values 1.0000e+20 replaced with NaN'];
D.metadata.readme = op_format_readme(readme);  %USE op_format_readme(readme) TO CONVERT.
D.metadata.user_friendly_time=datestr(ttime,'yyyy-mm-dd HH:MM:SS');
D.metadata.request_date = request.RequestDate;
D.metadata.request=request;
% split matrix into a series of variables opendap_nnnn
F=D;     
for k=1:length(D.time);
    if isfield(D,'nlwrs')==1 
        F.nlwrs=squeeze(D.nlwrs.nlwrs(:,:,k)); 
        F.metadata.nlwrs_variables={'latitude','longitude'};
    end
    if isfield(D,'nswrs')==1 
        F.nswrs=squeeze(D.nswrs.nswrs(:,:,k));
        F.metadata.nswrs_variables={'latitude','longitude'};
    end
    if isfield(D,'lhtfl')==1 
        F.lhtfl=squeeze(D.lhtfl.lhtfl(:,:,k)); 
        F.metadata.lhtfl_variables={'latitude','longitude'};
    end
    if isfield(D,'shtfl')==1 
        F.shtfl=squeeze(D.shtfl.shtfl(:,:,k)); 
        F.metadata.shtfl_variables={'latitude','longitude'};
    end
%   F.datetime=D.datetime(k,:);
    F.time=D.time(k);
    F.metadata.user_friendly_time=datestr(ttime(k),'yyyy-mm-dd HH:MM:SS');
    
    if R.SaveWorkspace == 'y' & strcmp(R.SaveMode,'opendap')==1    
    KF=num2str(k+kw0,'%04d');
    WSN=['opendap_' KF];
    assignin('base',WSN,F)
    end    
    if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
    % save extracted data to files
    KF=num2str(k+kf0,'%04d');
    FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
    FileName=FN3;
    save(FN3,'D')
    end

    end
end

if R.SaveFiles == 'y' & strcmp(R.SaveMode,'native')==1
% save extracted data to files
WSN=['OAFlux']; 
FN3=[R.FNPREFIX WSN '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3
save(FN3,'D')
end

xprogr=1; % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','OAFlux Download Progress');
close(hwaitbar)

%msgbox('Data have been acquired.')
disp('Data have been acquired.')
end %nargin
%==========================================================================
function [kw,kf]=LastFrameNum(R)
% find the latest saved opendap frame number 
%KF=num2str(kf,'%04d');
% the latest workspace frame number of form opendap_nnnn
a=evalin('base','who');k=strmatch('opendap_',a);
if isempty(k) k=0; 
else 
a=a{k(end)};
ki=strfind(a,'_');
    if length(ki)==1 ki(2)=length(a)+1; end
k=str2num(a(ki(1)+1:ki(2)-1));
end
if isempty(k) k=0;
end
kw=k;
% the latest saved file of form FNPREFIXnnnn.mat
if isempty(R.DIRNAME) R.DIRNAME='.'; end
%[R.DIRNAME '\' R.FNPREFIX '*.mat']
a=dir([R.DIRNAME '\' R.FNPREFIX '*.mat']);
a={a.name};
if isempty(a) k=0;
else
a=a{end};
k=str2num(a(length(R.FNPREFIX)+1:end-4));
end
if isempty(k) k=0;
end
kf=k;
