function varargout = get_pathfinder1km(varargin)
% Gets Pathfinder 1-km sea surface temperature data series.
%
% get_pathfinder1km
% get_pathfinder1km(request)
%
% INPUT(S):
%
%   request -- (optional) structure input which contains
%              information necessary to obtain data. The GUI
%              merely provides a graphical method of generating
%              the "request" variable. However, this visual
%              interface may be by-passed by this method.
%
% OUTPUT(S):
%
%   [none]
%
% OPeNDAP Science Team
% Copyright 2007
% $Version 2.0.0$

%==========================================================================
% Meri Sheremet
% REVISION HISTORY:
% 2007/09/25 created
% 2007/10/11 sst_raw added
% 2007/10/28 both Dec and Raw lists
%==========================================================================

% check for loaddap here before gui calls it
if ~exist('loaddap','file')
    errordlg('loaddap not found. Make sure it is in your MATLAB path.')
    return
end

if nargin < 1 || ~exist('gui_pathfinder1km_dataset.mat','file')
h = gui_pathfinder1km;
if nargout
    varargout{1} = h;
end
elseif nargin > 1
disp('Incorrect number of arguments.'); return  
else
request=varargin{1}; 
R=request;

if isempty(R.LAT1) | isempty(R.LAT2) | isempty(R.LON1) | isempty(R.LON2)
    errordlg(['Latitude/Longitude values are invalid or empty.']);
    return
end

DataSet=LoadRanges(R.Region);
Latitude=DataSet.Latitude;
Longitude=DataSet.Longitude;
Time=DataSet.Time;
iDec=DataSet.iDec;
iRaw=DataSet.iRaw;

% Find indices within the specified range of lats and lons
% Note that array Latitude runs from North to South
iLAT2=max(find(Latitude >= R.LAT1))-1;
iLAT1=min(find(Latitude <= R.LAT2))-1;
% Note that Longitude may jump from 180E to -179W in Western Pacific Region
i=find(Longitude >= R.LON1);[dummy,j]=min(Longitude(i));iLON1=i(j);iLON1=iLON1-1;
i=find(Longitude <= R.LON2);[dummy,j]=max(Longitude(i));iLON2=i(j);iLON2=iLON2-1;

if(isempty(iLAT1) | isempty(iLAT2)) | (iLAT2<iLAT1)
    errordlg(['Requested latitudes are out of range.']);
    return
end
if(isempty(iLON1) | isempty(iLON2)) | (iLON2<iLON1)
    errordlg(['Requested longitudes are out of range.']);
    return
end
if(isempty(R.LATINCR)|isempty(R.LONINCR)|isempty(R.DATEINCR)|R.LATINCR==0|R.LONINCR==0|R.DATEINCR==0) 
    errordlg(['Subsampling values are invalid.']);
    return
end
if(isempty(R.DATE1) | isempty(R.DATE2)) | (str2num(R.DATE2)<str2num(R.DATE1))
    errordlg(['Selected dates are invalid.']);
    return
end

JD1=datenum(R.DATE1,'yyyymmdd');
JD2=datenum(R.DATE2,'yyyymmdd');

JDA=datenum(Time,'yyyy-mm-dd HH:MM:SS');
JDB=JDA;
iDATE1=min(find(JDB >= JD1));
iDATE2=max(find(JDA <= JD2+1));

if(isempty(iDATE1) | isempty(iDATE2)) | (iDATE2<iDATE1)
    errordlg(['No data within the requested date range.']);
    return
end

[NTime,N2Time]=size(Time); %
iTime=(1:NTime)'; % pointer array  to elements in Time array
iTime=iTime(find(iTime >= iDATE1 & iTime <= iDATE2)); % limit by selected dates

% calculate time zone based on the range of longitudes
if strcmp(R.Region,'Northwest Atlantic') TZone=0.5*(-112-28)/15; end
if strcmp(R.Region,'Northeast Pacific' ) TZone=0.5*(-165-85)/15;  end
if strcmp(R.Region,'Northwest Pacific' ) TZone=0.5*(104+186)/15; end
HH=str2num(Time(iTime,12:13))+str2num(Time(iTime,15:16))/60; % hour of satellite pass, UTC
HH=mod(HH+TZone,24); % hour of satellite pass, local time.
HHNightDay=6; % boundary between night and day images, local time. 
if(R.Pass{1,2}=='n' & R.Pass{2,2}=='n') 
    errordlg(['No satellite passes selected.']);
    return
elseif R.Pass{1,2}=='n'
    j=find(HH>=HHNightDay & HH < HHNightDay+12);
    iTime=iTime(j); % exclude night passes
%    Time(iTime,:)
elseif R.Pass{2,2}=='n'
    j=find(HH<HHNightDay  | HH >= HHNightDay+12);
    iTime=iTime(j); % exclude day passes
%    Time(iTime,:)
end

if(isempty(iTime)) 
    errordlg(['No data within specified range of dates/passes.']);
    return
end

if R.SaveWorkspace=='n' & R.SaveFiles=='n'
    errordlg(['No saving mode selected.']);
    return
end
if length(R.DIRNAME) > 0
    if exist(R.DIRNAME)==0
        eval(['mkdir ' R.DIRNAME])
    end    
end

% Estimate the amount of requested data
MBLIMIT=100;
kV=0;
LF=size(R.Fields);
for k=1:LF(1)
    if R.Fields{k,2}=='y' kV=kV+1; end % number of variables
end
MB=round(1*(iLAT2-iLAT1)/R.LATINCR*(iLON2-iLON1)/R.LONINCR*length(iTime)/R.DATEINCR*kV*16/1024/1024);
if MB>MBLIMIT
ButtonName=questdlg(['The amount of data requested is ' num2str(MB) ' MB.'], ...
        'Pathfinder1km Requested Data Amount','Continue','Cancel','Cancel');
if strcmp(ButtonName,'Cancel') return; end
end

CLAT=['[' num2str(iLAT1) ':' num2str(R.LATINCR) ':' num2str(iLAT2) ']']; % ConstraintLatitude
CLON=['[' num2str(iLON1) ':' num2str(R.LONINCR) ':' num2str(iLON2) ']']; % ConstraintLongitude

if strcmp(R.SaveMode,'opendap')==1 
% find the latest saved opendap frame number
[kw0,kf0]=LastFrameNum(R);
end

URLBase='http://satdat1.gso.uri.edu/thredds/dodsC/';
if strcmp(R.Region,'Northwest Atlantic') URL1=[URLBase 'NWAtlantic']; end
if strcmp(R.Region,'Northeast Pacific' ) URL1=[URLBase 'NEPacific'];  end
if strcmp(R.Region,'Northwest Pacific' ) URL1=[URLBase 'NWPacific'];  end

% ************************************************************************
% alternative saving mode for user convenience
% save sst as a single 3D array 
% warning 3D array may not fit into computer memory
NT=length(iTime);
if strcmp(R.SaveMode,'opendap')==0
clear D3
L1=length((1:R.DATEINCR:NT));L2=length((iLAT1:R.LATINCR:iLAT2));L3=length((iLON1:R.LONINCR:iLON2));
% allocate memory for sst array
for kV=1:LF(1)
        if R.Fields{kV,2}=='y'
            field=R.Fields{kV,1};
eval(['D3.' field '=zeros(L1,L2,L3)*NaN;']);
        end
    end

end
% ************************************************************************

% Progress bar
hwaitbar=waitbar(0,'Acquiring ','Name','Pathfinder1km Download Progress');
% browse through dates
kf=0;
for k=1:R.DATEINCR:NT
kf=kf+1;    

clear D
% create standard blank opendap_nnnn data structure
for kV=1:LF(1)
        if R.Fields{kV,2}=='y'    
field=R.Fields{kV,1}; %
eval(['D.' field '=NaN;']);
        end
end
D.latitude=NaN;
D.longitude=NaN;
D.time=NaN;
for kV=1:LF(1)
        if R.Fields{kV,2}=='y'    
field=R.Fields{kV,1}; %
eval(['D.metadata.' field '_variables={''latitude'',''longitude''};']);
        end
end
D.metadata.url='';
D.metadata.attributes='';
D.metadata.name='pathfinder1km';
D.metadata.temporal='time_series';
D.metadata.reference_time=datestr([1970,01,01,00,00,00],'yyyy-mm-dd HH:MM:SS');
readme=['time in seconds since reference_time; /' ...
        'sst renamed from dsp_band_1, converted to physical units; /' ...
        'missing values replaced with NaN.'];
D.metadata.readme=op_format_readme(readme);  %USE op_format_readme(readme) TO CONVERT.
D.metadata.user_friendly_time='';
D.metadata.request_date=request.RequestDate;
D.metadata.request=request;
          
% sample loaddap request
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                               time   lat    lon
%loaddap('http://satdat1.gso.uri.edu/thredds/dodsC/NWAtlanticDec_1km?dsp_band_1[0:1:1][0:1:2][0:2:4]'

    for kV=1:LF(1)
        if R.Fields{kV,2}=='y'
% browse through checked variables    
field=R.Fields{kV,1}; %
field3=R.Fields{kV,3};  % short name Dec,Raw
eval(['iT=i' field3 '(iTime(k));']);
%iT
s=[field ' ' Time(iTime(k),:)];
s=strrep(s,'_',' ');
xprogr=(k-1)/NT; % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s ],'Name','Pathfinder1km Download Progress');

            if ~isnan(iT) % proceed with downloading data
CTIME=['[' num2str(iT-1) ']']; % Constraint Time download one image at at time
URL2=[URL1 field3 '_1km']; 
URL=[URL2 '?dsp_band_1'  CTIME CLAT CLON];
%URL3=URL;
disp(URL)
A=loaddap('-A',URL);
loaddap('+v',URL)

sst=dsp_band_1.dsp_band_1;
sst(find(sst<=3))=NaN; % replace missing values with NaN
scale=A.dsp_band_1.scale_factor;
offset=A.dsp_band_1.add_off;
sst=sst*scale+offset; % convert to physical values

eval(['D.' field '=sst;']);
% lat,lon,time are supposed to be the same for all fields
D.latitude=dsp_band_1.lat;
D.longitude=dsp_band_1.lon;
D.time=dsp_band_1.time;% in seconds since 1970-01-01 00:00:00
eval(['D.metadata.url.' field '=URL;']);
eval(['D.metadata.attributes.' field '=A;']);
            else % frame is missing 
eval(['D.' field '=NaN;']);
eval(['D.metadata.url.' field '='''';']);
eval(['D.metadata.attributes.' field '='''';']);
            end
        end
    end

Time70=D.time;
user_friendly_time=datestr(datenum(1970,1,1,0,0,0)+Time70/86400,'yyyy-mm-dd HH:MM:SS'); % Gregorian, char array 
D.metadata.user_friendly_time=user_friendly_time;
%D.metadata.user_friendly_time2=Time(iTime(k),:); %test

if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==1
    KF=num2str(kf+kw0,'%04d');
    WSN=['opendap_' KF];
    assignin('base',WSN,D)
end

if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==1
% save extracted data to files
KF=num2str(kf+kf0,'%04d');
FN3=[R.FNPREFIX KF '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D')
end

if strcmp(R.SaveMode,'opendap')==0
    for kV=1:LF(1)
        if R.Fields{kV,2}=='y'
            field=R.Fields{kV,1};
eval(['D3.' field '(kf,:,:)=D.' field ';']);
        end
    end
end

end %k iTime

% ************************************************************************
% alternative saving mode for user convenience
% save sst as a single 3D array 
% warning 3D array may not fit into computer memory
if strcmp(R.SaveMode,'opendap')==0
    D3.latitude=D.latitude;
    D3.longitude=D.longitude;
    D3.user_friendly_time=Time(iTime,:);
    D3.metadata.sst_variables={'user_friendly_time','latitude','longitude'};
    D3.metadata.name='Pathfinder1km';
    D3.metadata.request=request;
end
if R.SaveWorkspace=='y' & strcmp(R.SaveMode,'opendap')==0
assignin('base','Pathfinder1km',D3)
end
if R.SaveFiles == 'y' & strcmp(R.SaveMode,'opendap')==0
% save extracted data to files
FN3=[R.FNPREFIX 'Pathfinder1km' '.mat'];
    if length(R.DIRNAME) > 0
    FN3=[R.DIRNAME '/' FN3];
    end
FileName=FN3;
save(FN3,'D3')
end
% ************************************************************************

xprogr=1; % progress fraction
waitbar( xprogr , hwaitbar, ['Acquiring ' s],'Name','Pathfinder1km Download Progress');
close(hwaitbar)
disp('Data have been acquired.')
end % nargin

function [kw,kf]=LastFrameNum(R)
% find the latest saved opendap frame number 
%KF=num2str(kf,'%04d');
% the latest workspace frame number of form opendap_nnnn
a=evalin('base','who');k=strmatch('opendap_',a);
if isempty(k) k=0; 
else 
a=a{k(end)};
ki=strfind(a,'_');
    if length(ki)==1 ki(2)=length(a)+1; end
k=str2num(a(ki(1)+1:ki(2)-1));
end
if isempty(k) k=0;
end
kw=k;
% the latest saved file of form FNPREFIXnnnn.mat
if isempty(R.DIRNAME) R.DIRNAME='.'; end
%[R.DIRNAME '\' R.FNPREFIX '*.mat']
a=dir([R.DIRNAME '\' R.FNPREFIX '*.mat']);
a={a.name};
if isempty(a) k=0;
else
a=a{end};
k=str2num(a(length(R.FNPREFIX)+1:end-4));
end
if isempty(k) k=0;
end
kf=k;

function DataSet=LoadRanges(Region)
if ~exist('gui_pathfinder1km_dataset.mat','file') 
    errordlg(['File gui_pathfinder1km_dataset.mat does not exist.',...
        'Reissue ',mfilename,'.']);
    return
end
load gui_pathfinder1km_dataset.mat
if strcmp(Region,'Northwest Atlantic') DS=['NWAtlantic']; end
if strcmp(Region,'Northeast Pacific' ) DS=['NEPacific' ]; end
if strcmp(Region,'Northwest Pacific' ) DS=['NWPacific' ]; end
eval(['DataSet=DS' DS ';'])