function [msg,err,request]=get_seawinds_interimdatacheck(request)
% This "interimdatacheck" corresponds to QUIKSCAT and ADEOS II data.
%
% This function checks the user's inputs to make sure they makes sense.
%
% Christian Buckingham
% Meri Sheremet
% OPeNDAP Project, 2006

% Parse vector.
DATE1 = request.DATE1;
DATE2 = request.DATE2;
DATEINC = request.DATEINC;
LAT1 = request.LAT1;
LAT2 = request.LAT2;
LATINC = request.LATINC;
LON1 = request.LON1;
LON2 = request.LON2;
LONINC = request.LONINC;
DataType = request.DataType;
Field = request.Field;
Pass = request.Pass;
SaveWorkspace = request.SaveWorkspace;
SaveWorkspaceMode = request.SaveWorkspaceMode;
SaveFile = request.SaveFile;
directory = request.DIRNAME;
prefx = request.FNPREFIX;

%% Check to make sure certain inputs exist.
if (~DataType.value(1) && ~DataType.value(2)) %instrument2
    err=1;
    msg='An instrument (QuikSCAT, ADEOS II) has not been selected. Please correct and try again.';
    return
end
len=length(Field.value);
if (sum(Field.value(1:len))==0)
    err=1;
    msg='Parameters (Wind Vectors, Winds Stress) have not been selected. Please correct and try again.';
    return
end
if (isempty(DATE1) || isempty(DATE2))
    err=1;
    msg='A date range has not been entered. Please correct and try again.';
    return
end
if (length(DATE1)~=8)
    err=1;
    msg='The chosen dates have not been entered correctly. Please correct and try again.';
    return
end
if (length(DATE2)~=8)
    err=1;
    msg='The chosen dates have not been entered correctly. Please correct and try again.';
    return
end
len=length(Pass.value);
if (sum(Pass.value(1:len))==0)
    err=1;
    msg='A pass (ascending, descending) has not been selected. Please correct and try again.';
    return
end
if (isempty(LAT1) || isempty(LAT2))
    err=1;
    msg='A latitude range has not been entered. Please correct and try again.';
    return
end
if (isempty(LON1) || isempty(LON2))
    err=1;
    msg='A longitude range has not been entered. Please correct and try again.';
    return
end
if (isempty(DATEINC))
    err=1;
    msg='A time step has not been entered. Please correct and try again.';
    return
end
if (isempty(LATINC))
    err=1;
    msg='A latitude step has not been entered. Please correct and try again.';
    return
end
if (isempty(LONINC))
    err=1;
    msg='A longitude step has not been entered. Please correct and try again.';
    return
end
if (~SaveWorkspace && ~SaveFile)
    err=1;
    msg='An output type has not been selected. Please choose an ouput type.';
    return
end

%% DATE1, DATE2.
% Instrument dates.
disp('Please wait. Acquiring metadata ...')
[msg,err,START_INSTRUMENT1,END_INSTRUMENT1]=qscat_get_dates;
if (err)
    START_INSTRUMENT1='19990719';
    END_INSTRUMENT1=strrep(datestr(now,29),'-','');
end
[msg,err,START_INSTRUMENT2,END_INSTRUMENT2]=adeos_get_dates;
if (err)
    START_INSTRUMENT2='20030410';
    END_INSTRUMENT2='20031024';
end
if (err)
    msg = ['We cannot make a connection to server. ',...
        'We will use a default set of dates.'];
    disp(msg);
end
disp('Connection complete.')

% User dates.
date1vec=[str2num(DATE1(1:4)),str2num(DATE1(5:6)),str2num(DATE1(7:8))];
date2vec=[str2num(DATE2(1:4)),str2num(DATE2(5:6)),str2num(DATE2(7:8))];
date1=datenum(date1vec);
date2=datenum(date2vec);

% Instrument dates revisited.
start1vec=[str2num(START_INSTRUMENT1(1:4)),str2num(START_INSTRUMENT1(5:6)),str2num(START_INSTRUMENT1(7:8))];
start2vec=[str2num(START_INSTRUMENT2(1:4)),str2num(START_INSTRUMENT2(5:6)),str2num(START_INSTRUMENT2(7:8))];
end1vec=[str2num(END_INSTRUMENT1(1:4)),str2num(END_INSTRUMENT1(5:6)),str2num(END_INSTRUMENT1(7:8))];
end2vec=[str2num(END_INSTRUMENT2(1:4)),str2num(END_INSTRUMENT2(5:6)),str2num(END_INSTRUMENT2(7:8))];
start1=datenum(start1vec);
start2=datenum(start2vec);
end1=datenum(end1vec);
end2=datenum(end2vec);

% Compare dates.
err=0;
msg='';
if (DataType.value(1)) %instrument1
ok_main_a = (start1 <= date1) && (date1 <= end1); %date1 contained in data record.
ok_main_b = (start1 <= date2) && (date2 <= end1); %date2 contained in data record.
ok_main_c = (date1 <= date2);
end
if (DataType.value(2)) %instrument2
ok_adeos_a = (start2 <= date1) && (date1 <= end2); %date1 contained in instrument2 data record.
ok_adeos_b = (start2 <= date2) && (date2 <= end2); %date2 contained in instrument2 data record.
ok_main_c = (date1 <= date2);
end

if (DataType.value(1)) %instrument1
err_date = ~(ok_main_a && ok_main_b && ok_main_c);
if err_date
    err=1;
    msg='Dates entered are not valid. Please correct and re-enter.';
    return
end
end
if (DataType.value(2)) %instrument2
ok_adeos = (ok_adeos_a && ok_adeos_b && ok_main_c);
if ~ok_adeos
    err=1;
    msg='Dates entered are not valid. Please correct and re-enter.';
    return
end
end

%% LAT1, LAT2, LON1, LON2.
start1 = -90; %latitude
start2 = 0;   %longitude
end1 = +90;   %latitude
end2 = 360;   %longitude
lat1=str2num(LAT1);
lat2=str2num(LAT2); 
lon1=str2num(LON1);
lon2=str2num(LON2);
if (lon1<=-180),
    lon1=-180+.125;
    LON1='-180';
    %LON1=num2str(-180);
end
if (lon2>=360),
    lon2=360-.125;
    LON2='360';
    %LON2=num2str(lon2);
end
lon1=mod(lon1,360);
lon2=mod(lon2,360);

ok_main_lats_a = (start1 <= lat1) && (lat1 <= end1);
ok_main_lats_b = (start1 <= lat2) && (lat2 <= end1);
ok_main_lats_c = lat1 < lat2;
ok_main_lons_a = (start2 <= lon1) && (lon1 <= end2);
ok_main_lons_b = (start2 <= lon2) && (lon2 <= end2);
ok_main_lons_c = lon1 < lon2;

err_lats = ~(ok_main_lats_a && ok_main_lats_b && ok_main_lats_c);
err_lons = ~(ok_main_lons_a && ok_main_lons_b && ok_main_lons_c);
if (err_lats)
    err=1;
    msg='Latitudes entered are not valid. Please correct and re-enter.';
    return
end
if (err_lons)
    err=1;
    msg='Longitudes entered are not valid. Please correct and re-enter.';
    return
end

%% DATEINC, LATINC, LONINC.
ok_timeinc = (rem(str2num(DATEINC),1)==0);
ok_latinc  = (rem(str2num(LATINC),1)==0);
ok_loninc  = (rem(str2num(LONINC),1)==0);

if (~ok_timeinc)
    err=1;
    msg='Time step must be an integer. Please correct and re-enter.';
    return
end
if (~ok_latinc)
    err=1;
    msg='Latitude step must be an integer. Please correct and re-enter.';
    return
end
if (~ok_loninc)
    err=1;
    msg='Longitude step must be an integer. Please correct and re-enter.';
    return
end

%% SaveWorkspace, SaveFile, Other.
ok_save = SaveWorkspace | SaveFile;
if ~(ok_save)
    err=1;
    msg='Data must be saved in some fashion. Please select how your data is to be saved.';
    return
end

% Request Vector.
request = struct('DATE1',DATE1,...
                 'DATE2',DATE2,...
                 'DATEINC',DATEINC,...
                 'LAT1',LAT1,...
                 'LAT2',LAT2,...
                 'LATINC',LATINC,...
                 'LON1',LON1,...
                 'LON2',LON2,...
                 'LONINC',LONINC,...
                 'DataType',DataType,...
                 'Field',Field,...
                 'Pass',Pass,...
                 'SaveWorkspace',SaveWorkspace,...
                 'SaveWorkspaceMode',SaveWorkspaceMode,...
                 'SaveFile',SaveFile,...
                 'DIRNAME',directory,...
                 'FNPREFIX',prefx);
    
return %endoffunction
%==========================================================================